--author: igor29381
--v1.0
--date: 05.12.17

PaintingAge = {};

function PaintingAge.prerequisitesPresent(specializations)
	return true;
end;

function PaintingAge:load(savegame)
	self.paintingAge = {state = 1, wasMoved = false, nodes = {}};
	self.workingParts = {state = 1, sentState = 1, nodes = {}};
	if hasXMLProperty(self.xmlFile, "vehicle.paintingAge") then
		self.hourChanged = PaintingAge.hourChanged;
		self.updatePaintingNodes = PaintingAge.updatePaintingNodes;
		self.updateWorkingParts = PaintingAge.updateWorkingParts;
		self.paintingAgeDirtyFlag = self:getNextDirtyFlag();
		self.paintingAge.delta = 1/(Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.paintingAge#duration"), 50)*24);
		self.workingParts.duration = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.paintingAge#workingPartsDuration"), 5)*60000;
		local i = 0;
		while true do
			local baseString = string.format("vehicle.paintingAge.paintingNode(%d)",i);
			local index = getXMLString(self.xmlFile, baseString.."#index");
			local workingPart = Utils.getNoNil(getXMLBool(self.xmlFile, baseString.."#workingPart"), false);
			if index == nil then
				break;
			end;
			local node = Utils.indexToObject(self.components, index);
			if workingPart then
				table.insert(self.workingParts.nodes, node);
			else
				table.insert(self.paintingAge.nodes, node);
			end;
			i=i+1;
		end;
		if self.isServer then
			g_currentMission.environment:addHourChangeListener(self);
		end;
	end;
	if savegame then
		self.paintingAge.state = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. "#paintingAgeState"), 1);
		self:updatePaintingNodes();
		self.workingParts.state = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. "#workingPartsState"), 1);
		self:updateWorkingParts();
	end;
end;

function PaintingAge:getSaveAttributesAndNodes(nodeIdent)
	local attributes = '';
    local nodes = "";
	attributes = attributes..'paintingAgeState="'..self.paintingAge.state..'" workingPartsState="'..self.workingParts.state..'"';
    return attributes, nodes;
end;

function PaintingAge:delete()
	g_currentMission.environment:removeHourChangeListener(self);
end;

function PaintingAge:readStream(streamId, connection)
	self.paintingAge.state = streamReadInt32(streamId)*0.001;
	self.workingParts.state = streamReadInt32(streamId)*0.001;
	self:updatePaintingNodes();
	self:updateWorkingParts();
end;

function PaintingAge:writeStream(streamId, connection)
	streamWriteInt32(streamId, math.floor(self.paintingAge.state*1000));
	streamWriteInt32(streamId, math.floor(self.workingParts.state*1000));
end;

function PaintingAge:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then
        local dirty = streamReadBool(streamId);
        if dirty then
			self.paintingAge.state = streamReadInt32(streamId)*0.001;
			self.workingParts.state = streamReadInt32(streamId)*0.001;
			self:updatePaintingNodes();
			self:updateWorkingParts();
        end;
    end;
end;

function PaintingAge:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() and self.washableNodes ~= nil then
        if streamWriteBool(streamId, bitAND(dirtyMask, self.paintingAgeDirtyFlag) ~= 0) then
            streamWriteInt32(streamId, math.floor(self.paintingAge.state*1000));
			streamWriteInt32(streamId, math.floor(self.workingParts.state*1000));
        end;
    end;
end;

function PaintingAge:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PaintingAge:keyEvent(unicode, sym, modifier, isDown)
end;

function PaintingAge:update(dt)
	if self:getLastSpeed(true) > 5 then
		if self.isServer then
			if not self.paintingAge.wasMoved then
				self.paintingAge.wasMoved = true;
			end;
			if self.workingParts.state > 0 then
				if (self.sowingMachineHasGroundContact or self.cultivatorHasGroundContact or self.ploughHasGroundContact) and self.movingDirection > 0 then
					local delta = 1/self.workingParts.duration*dt;
					self.workingParts.state = math.max(self.workingParts.state - delta, 0);
					self:updateWorkingParts();
					if math.abs(self.workingParts.sentState - self.workingParts.state) > 0.01 then
						self:raiseDirtyFlags(self.paintingAgeDirtyFlag);
						self.workingParts.sentState = self.workingParts.state;
					end;
				end;
			end;
		end;
	end;
end;

function PaintingAge:updateTick(dt)
end;

function PaintingAge:draw()
end;

function PaintingAge:hourChanged()
	if self.isServer then
		local sentState = false;
		if self.paintingAge.state > 0 then
			local delta = self.paintingAge.delta;
			if self.paintingAge.wasMoved then
				delta = delta*1.1;
				self.paintingAge.wasMoved = false;
			end;
			self.paintingAge.state = math.max(self.paintingAge.state - delta, 0);
			self:updatePaintingNodes();
			sentState = true;
		end;
		if self.workingParts.state < 1 then
			self.workingParts.state = math.min(self.workingParts.state + 0.02, 1);
			self:updateWorkingParts();
			sentState = true;
		end;
		if sentState then
			self:raiseDirtyFlags(self.paintingAgeDirtyFlag);
		end;
	end;
end;

function PaintingAge:updatePaintingNodes()
	if self.paintingAge.nodes and #self.paintingAge.nodes > 0 then
		for i=1, #self.paintingAge.nodes do
			local node = self.paintingAge.nodes[i];
			local x,y,z,w = getShaderParameter(node, "RDT");
			setShaderParameter(node, "RDT", self.paintingAge.state, y, z, w, false);
		end;
	end;
end;

function PaintingAge:updateWorkingParts()
	if self.workingParts.nodes and #self.workingParts.nodes > 0 then
		for i=1, #self.workingParts.nodes do
			local node = self.workingParts.nodes[i];
			local x,y,z,w = getShaderParameter(node, "RDT");
			setShaderParameter(node, "RDT", self.workingParts.state, y, z, w, false);
		end;
	end;
end;
